#include "CudaDevice.h"

extern "C" 
{
	void CUDADeviceInit(int dev) 
	{
		int count;
		cudaDeviceProp p;
		cudaGetDeviceCount(&count);

		for(int i = 0; i < count; ++i)
		{
			cudaGetDeviceProperties(&p, i);

			printf("#%d\n", i);
			printf("  Name:				%s\n", p.name );
			printf("  Revision:			%d.%d\n", p.major, p.minor );
			printf("  Global Mem:			%u\n", p.totalGlobalMem );
			printf("  Shared/Blk:			%d\n", p.sharedMemPerBlock );
			printf("  Regs/Blk:			%d\n", p.regsPerBlock );
			printf("  Warp Size:			%d\n", p.warpSize );
			printf("  Mem Pitch:			%d\n", p.memPitch );
			printf("  Thrds/Blk:			%d\n", p.maxThreadsPerBlock );
			printf("  Max block dims:		(%d ,%d,%d)\n", p.maxThreadsDim[0], p.maxThreadsDim[1], p.maxThreadsDim[2]);
			printf("  Max grid dims:		(%d,%d,%d)\n", p.maxGridSize[0], p.maxGridSize[1], p.maxGridSize[2]);
			printf("  Const Mem:			%d\n", p.totalConstMem );
			printf("  Clock Rate:			%d\n", p.clockRate );
			printf("  Can map host memory:		%d\n", p.canMapHostMemory );
		}

		if(dev >= count || dev < 0)
			dev = cutGetMaxGflopsDeviceId();
	    
		printf("Setting device...\n");

		cudaSetDevice(dev);
		cudaGLSetGLDevice(dev);
		
		printf("Initialized using device: %d\n", dev);
	}

	void pboRegister(int pbo)
	{
		cudaGLRegisterBufferObject(pbo);
	}

	void pboUnregister(int pbo)
	{
		cudaGLUnregisterBufferObject(pbo);
	}

} // extern "C"