#include "cudaTransform.h"
#include "cudaTransformKernels.cu"
#include <GL/glew.h>
#include <GL/glfw.h>
#include <cuda_gl_interop.h>
#include <cutil.h>


float tIn[6];
float *transformInput;
unsigned char *in_data;
unsigned char * out_data;

extern "C" 
{
	void CUDATransformInit(int n)
	{		
		// GPU memory allocation
		cudaMalloc(&transformInput, 6 * sizeof(float));
	}

	void CUDATransformFree()
	{
		// Free GPU memory
		cudaFree(transformInput);
		printf("device memory freed \n");
	}

	void CUDATransformPixels(int pbo_in, int pbo_out, point3 cop, point3 p)
	{
		// Input parameters
		tIn[0] = cop.x;		//xo
		tIn[1] = cop.y;		//yo
		tIn[2] = cop.z;		//zo
		tIn[3] = p.x;		//xj
		tIn[4] = p.y;		//yj
		tIn[5] = p.z;		//zj

		// Store parameters in device memory
		cudaMemcpy(transformInput, tIn, 6 * sizeof(float), cudaMemcpyHostToDevice);

		CUDA_SAFE_CALL(cudaGLMapBufferObject( (void**)&in_data, pbo_in));
		CUDA_SAFE_CALL(cudaGLMapBufferObject( (void**)&out_data, pbo_out));
	

		// block and grid dimensions
		int numBlocks1 = 1;
		dim3 gridDim1(numBlocks1,1,1);
		dim3 blockDim1(WIDTH/numBlocks1,1,1);
		
		dim3 blockDim2(16,8,1);
		int numBlocks2 = (WIDTH * (HEIGHT/2)) / (16 * 8);
		dim3 gridDim2(numBlocks2,1,1);

		dim3 blockDim3(16,16,1);
		int numBlocks3 = (WIDTH * (HEIGHT)) / (16 * 16);
		dim3 gridDim3(numBlocks3,1,1);

		dim3 blockDim4(32,32,1);
		int numBlocks4 = (WIDTH * (HEIGHT/2)) / (32 * 32);
		dim3 gridDim4(numBlocks4,1,1);
		
		// Transform
		//CylinderTransformKernel<<<gridDim1, blockDim1>>>(transformInput, pixelsIn, pixelsOut);
		//cudaThreadSynchronize();
		CylinderTransformKernel2<<<gridDim2, blockDim2>>>(transformInput, in_data, out_data);
		//cudaThreadSynchronize();
		//SphereTransformKernel<<<gridDim1, blockDim1>>>(transformInput, pixelsIn, pixelsOut);
		//cudaThreadSynchronize();
		SphereTransformKernel2<<<gridDim2, blockDim2>>>(transformInput, in_data, out_data);
		//cudaThreadSynchronize();
		//CombinedTransformKernel<<<gridDim1, blockDim1>>>(transformInput, pixelsIn, pixelsOut);
		//cudaThreadSynchronize();
		//CombinedTransformKernel2<<<gridDim3, blockDim3>>>(transformInput, in_data, out_data);

		//pboTest<<<gridDim3, blockDim3>>>(transformInput, in_data, out_data);
		
		cudaGLUnmapBufferObject( pbo_in);
		cudaGLUnmapBufferObject( pbo_out);
	}
}