#include "main.h"

/*******************
Change COP position
*******************/
void moveCop(floattype dx, floattype dy, floattype dz)
{
  cop.x += dx;
  cop.y += dy;
  cop.z += dz;
}



/********************
Handle keyboard input
********************/
void GLFWCALL keyHandler(int key, int action) {
	if(action == GLFW_PRESS) {
		switch(key) {
			case 'A':
				moveCop(0.05,0.0,0.0);
				break;
			case 'D':
				moveCop(-0.05,0.0,0.0);
				break;
			case 'W':
				moveCop(0.0,-0.05,0.0);
				break;
			case 'S':
				moveCop(0.0,0.05,0.0);
				break;
			case 'C':
				moveCop(0.0,0.0,0.05);
				break;
			case 'V':
				moveCop(0.0,0.0,-0.05);
				break;
			case 'J':
				moveObj(-0.05,0.0,0.0);
				break;
			case 'L':
				moveObj(0.05,0.0,0.0);
				break;
			case 'I':
				moveObj(0.0,0.05,0.0);
				break;
			case 'K':
				moveObj(0.0,-0.05,0.0);
				break;
			case 'N':
				moveObj(0.0,0.0,0.05);
				break;
			case 'M':
				moveObj(0.0,0.0,-0.05);
				break;
			case 'O':
				stereo = !stereo;
				break;
			case 'T':
				transform = !transform;
				break;
			default:
				break;
		}
	}
}



/***********************
Initialize stencil mask
For interlaced stereoscopic rendering
***********************/
void glEnable2D()
{
	int vPort[4];
	glGetIntegerv(GL_VIEWPORT, vPort);

	glMatrixMode(GL_PROJECTION);
	glPushMatrix();
	glLoadIdentity();
	glOrtho(0, vPort[2], 0, vPort[3], -1, 1);

	glMatrixMode(GL_MODELVIEW);
	glPushMatrix();
	glLoadIdentity();
}

void glDisable2D()
{
   glMatrixMode(GL_PROJECTION);
   glPopMatrix();   
   glMatrixMode(GL_MODELVIEW);
   glPopMatrix();	
}

void initStencilMask()
{
	glClearStencil(0);
	glClear(GL_STENCIL_BUFFER_BIT);
	glEnable(GL_STENCIL_TEST);
	glStencilFunc(GL_ALWAYS, 1, 1);
	glStencilOp(GL_KEEP, GL_KEEP, GL_REPLACE);

	glDisable(GL_DEPTH_TEST);
	glEnable2D();

	// drawing stencil pattern
	glBegin(GL_LINES);
	for(int i = 0; i < HEIGHT/2; ++i) {
		glVertex2i(0, i*2);
		glVertex2i(WIDTH-1, i*2);
    }
    glEnd();

    glDisable2D();
	glEnable(GL_DEPTH_TEST);
	glDisable(GL_STENCIL_TEST);
    glStencilOp(GL_KEEP, GL_KEEP, GL_KEEP);
}



/***********************
Create/delete/display PBO and textures
For transfering image from OpenGL to CUDA
***********************/
void createPBO( GLuint* pbo)
{
    // set up vertex data parameter
    num_texels = WIDTH * HEIGHT;
    num_values = num_texels * 4;
    size_tex_data = sizeof(GLubyte) * num_values;
    void *data = malloc(size_tex_data);

    // create buffer object
    glGenBuffers( 1, pbo);
    glBindBuffer( GL_ARRAY_BUFFER, *pbo);

    // buffer data
    glBufferData( GL_ARRAY_BUFFER, size_tex_data, data, GL_DYNAMIC_DRAW);
    free(data);

    glBindBuffer( GL_ARRAY_BUFFER, 0);

    // attach this Buffer Object to CUDA
    pboRegister(*pbo);
}

void deletePBO( GLuint* pbo)
{
    glBindBuffer( GL_ARRAY_BUFFER, *pbo);
    glDeleteBuffers( 1, pbo);

    *pbo = 0;
}

void createTexture( GLuint* tex_name, unsigned int size_x, unsigned int size_y)
{
    // create a tex as attachment
    glGenTextures( 1, tex_name);
    glBindTexture( GL_TEXTURE_2D, *tex_name);

    // set basic parameters
    glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
    glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
    glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
    glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);

    // buffer data
    glTexImage2D( GL_TEXTURE_2D, 0, GL_RGBA, size_x, size_y, 0, GL_RGBA, GL_UNSIGNED_BYTE, NULL);
}
void deleteTexture( GLuint* tex)
{
    glDeleteTextures( 1, tex);
    *tex = 0;
}
void displayImageFromTexture(int stencilState)
{
    // render a screen sized quad
    glDisable(GL_DEPTH_TEST);
    glDisable(GL_LIGHTING);
    glEnable(GL_TEXTURE_2D);
    glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE);

    glMatrixMode( GL_PROJECTION);
    glPushMatrix();
		glLoadIdentity();
		glOrtho(-1.0, 1.0, -1.0, 1.0, -1.0, 1.0);

		glMatrixMode( GL_MODELVIEW);
		glLoadIdentity();

		glViewport(0, 0, WIDTH, HEIGHT);

		glEnable(GL_STENCIL_TEST);
		if(stencilState==1)
			glStencilFunc(GL_EQUAL, 1, 1);
		else if(stencilState==0)
			glStencilFunc(GL_NOTEQUAL, 1, 1);
		else
			glDisable(GL_STENCIL_TEST);
		glBegin( GL_QUADS);

			glTexCoord2f( 0.0, 0.0);
			glVertex3f( -1.0, -1.0, 0.5);

			glTexCoord2f( 1.0, 0.0);
			glVertex3f(  1.0, -1.0, 0.5);

			glTexCoord2f( 1.0, 1.0);
			glVertex3f(  1.0,  1.0, 0.5);

			glTexCoord2f( 0.0, 1.0);
			glVertex3f( -1.0,  1.0, 0.5);

		glEnd();
		glDisable(GL_STENCIL_TEST);

		glMatrixMode( GL_PROJECTION);
    glPopMatrix();

    glDisable( GL_TEXTURE_2D);
	glEnable(GL_DEPTH_TEST);
	if(LIGHTING)
		glEnable(GL_LIGHTING);

    glBindBuffer( GL_PIXEL_PACK_BUFFER_ARB, 0);
    glBindBuffer( GL_PIXEL_UNPACK_BUFFER_ARB, 0);
}



/***********************
Initialize program
***********************/
void init(int stereoMode, int transformation)
{
	/* GLFW init and setup */
	if (glfwInit() != GL_TRUE)
	{
		std::cout<<"glfw initialization failed.\n";
		shutDown(1);
	}
	
    #if QUAD_STEREO
            glfwOpenWindowHint(GLFW_STEREO, GL_TRUE);
    #endif
	
	glfwOpenWindowHint(GLFW_WINDOW_NO_RESIZE, GL_TRUE);
	if (glfwOpenWindow(WIDTH, HEIGHT, 8, 8, 8, 8, 16, 1, GLFW_WINDOW) != GL_TRUE)
	{
		std::cout<<"opening window failed.\n";
		shutDown(1);
	}
	glfwSetWindowTitle("TRANSFORM!");
	glfwSetWindowPos(100, 100);
	glfwSetKeyCallback(keyHandler);
	glfwEnable(GLFW_KEY_REPEAT);

	if(glewInit() != GLEW_OK)	// Glew init
	{
		std::cout<<"GLEW initialization failed. \n";
		shutDown(1);
	}


	/* Set CUDA device */
	CUDADeviceInit(0);			


	/* Init Color, lights and shading */
	glClearColor(0.0,0.0,0.0,0.0);
	glEnable(GL_DEPTH_TEST);
	glShadeModel(GL_SMOOTH);
	
	#if LIGHTING
		glEnable(GL_LIGHTING);
		glEnable(GL_LIGHT0);

		GLfloat ambient[] = { 0.5f, 0.5f, 0.5f, 1.0f };
		GLfloat diffuseLight[] = { 1.0f, 1.0f, 1.0, 1.0f };
		GLfloat position[] = { 2.0f, 2.0f, -1.0f, 1.0f };
		//GLfloat ambient[] = { 0.2f, 0.2f, 0.1f, 1.0f };
		//GLfloat position[] = { 2.0f, 6.0f, -2.5f, 1.0f };

		glLightfv(GL_LIGHT0, GL_AMBIENT, ambient);
		glLightfv(GL_LIGHT0, GL_DIFFUSE, diffuseLight);
		glLightfv(GL_LIGHT0, GL_POSITION, position);
	#endif


	/* Setup Projection mode */
	glViewport(0,0,WIDTH,HEIGHT);
	glMatrixMode(GL_PROJECTION);
	
	glLoadIdentity();
	glFrustum(-WIDTH_SCALE_FACTOR,  WIDTH_SCALE_FACTOR, 
			  -HEIGHT_SCALE_FACTOR, HEIGHT_SCALE_FACTOR, 
			  cop.z, 50);
	
	glMatrixMode(GL_MODELVIEW);


	/* Create pbos and texture */
    createPBO( &pbo_source);
    createPBO( &pbo_dest);
	createPBO( &pbo_dest2);
    createTexture( &tex_screen, WIDTH, HEIGHT);


	/* Other */
	initStencilMask();
	running = 1;
	stereo = stereoMode;
	transform = transformation;
	avgTransformTime = 0.0;

	

	/* Allocate memory and initialize transformations */
	#if (METHOD==1)
		std::cout<<"CUDA Image Transform\n";
		CUDATransformInit(WIDTH*HEIGHT*NUM_COLOR_FACTOR);
	#elif (METHOD==2)
		std::cout<<"Shaders Projection Transform\n";
		bool shaderStatus = transformShader.load("vTransform.vert", "vTransform.frag");
		if(!shaderStatus)
		{
			std::cout<<"Can't find shaders\n";
			shutDown(1);
		}
	#else
		std::cout<<"CPU Image Transform\n";
		pixels_in =  (unsigned char*)calloc(WIDTH*HEIGHT*NUM_COLOR_FACTOR, sizeof(unsigned char));
		pixels_out = (unsigned char*)calloc(WIDTH*HEIGHT*NUM_COLOR_FACTOR, sizeof(unsigned char));
		pixels_out2 = (unsigned char*)calloc(WIDTH*HEIGHT*NUM_COLOR_FACTOR, sizeof(unsigned char));
	#endif
}



/***********************
Shut down program
***********************/
void shutDown(int returnCode)
{
	glfwTerminate();
	std::cout<<"\nProgram terminated \n";

	#if (METHOD==1)
		CUDATransformFree();
	#endif

	exit(returnCode);
}



/***********************
Main loop of the program
***********************/
void mainLoop()
{
	double currentTime, deltaTime;
	double oldTime = glfwGetTime();
	int frameCount = 0;
	int numFrames = 0;
	int stopAfterXFrames = 0;//1000;
	double stopAfterXTime = 2*60;

	while(running)
	{
		/* Press Esc or close window to exit loop */
		running = !glfwGetKey(GLFW_KEY_ESC) && glfwGetWindowParam(GLFW_OPENED);

		/* Calculating FPS */
		currentTime = glfwGetTime();
		deltaTime = currentTime - oldTime;
		if(deltaTime >  1.0)
		{
			std::cout<<"fps: "<<frameCount/deltaTime<<" \n";
			numFrames += frameCount;
			frameCount = 0;
			oldTime = currentTime;
		}
		frameCount++;

		/* Stop for x number of frames */
		if(stopAfterXFrames>0)
		{
			if((numFrames + frameCount) >= stopAfterXFrames)
			{
				std::cout<<"Stopped after "<<stopAfterXFrames<<" frames.\n";
				running = false;
			}
		}

		/* Stop for x number of seconds */
		if(stopAfterXTime>0.0)
		{
			if((glfwGetTime()) >= stopAfterXTime)
			{
				std::cout<<"Stopped after "<<stopAfterXTime<<" seconds.\n";
				running = false;
			}
		}

		/* Draw frame */
		#if (METHOD==1)
			transformationCUDA(drawTeapot);
		#elif (METHOD==2)
			transformationShaders(drawTeapot);
		#else
			transformationCPU(drawBox);
		#endif

		/* Animate scene */
		//moveBox();
		//moveCube();
		moveTeapot();

	}

	/* Print statistics */
	std::cout<<"Jumped out of loop \n";
	std::cout<<"Total number of frames: "<<(numFrames + frameCount)<<"s \n";
	std::cout<<"Total elasped time: "<<glfwGetTime()<<"s \n";
	std::cout<<"Average fps : "<<((numFrames + frameCount)/glfwGetTime())<<"\n";
	std::cout<<"Average transform method runtime : "<<(avgTransformTime/(numFrames + frameCount))<<"\n";
}



/****************************************************
Render scene using the parallel pixel transformation
****************************************************/
void transformationCUDA(PF draw)
{
	float eyeDist = 0.1;
	double tStart;
	if(stereo)
	{
		#if QUAD_STEREO
			glDrawBuffer(GL_BACK_LEFT);
            glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
			glMatrixMode( GL_MODELVIEW);
			glLoadIdentity();
			glTranslated(cop.x-eyeDist, cop.y, cop.z);
			draw();

			pboUnregister(pbo_source);
			glBindBuffer( GL_PIXEL_PACK_BUFFER_ARB, pbo_source);
			glReadPixels( 0, 0, WIDTH, HEIGHT, GL_RGB, GL_UNSIGNED_BYTE, NULL);
			pboRegister(pbo_source);

			CUDATransformPixels(pbo_source, pbo_dest, cop, p);

			glBindBuffer( GL_PIXEL_UNPACK_BUFFER_ARB, pbo_dest);
			glBindTexture( GL_TEXTURE_2D, tex_screen);
			glTexSubImage2D( GL_TEXTURE_2D, 0, 0, 0, 
							WIDTH, HEIGHT, GL_RGB, GL_UNSIGNED_BYTE, NULL);

			displayImageFromTexture(2);

			glDrawBuffer(GL_BACK_RIGHT);
            glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
			glMatrixMode( GL_MODELVIEW);
			glLoadIdentity();
			glTranslated(cop.x+eyeDist, cop.y, cop.z);
			draw();

			pboUnregister(pbo_source);
			glBindBuffer( GL_PIXEL_PACK_BUFFER_ARB, pbo_source);
			glReadPixels( 0, 0, WIDTH, HEIGHT, GL_RGB, GL_UNSIGNED_BYTE, NULL);
			pboRegister(pbo_source);
	
			CUDATransformPixels(pbo_source, pbo_dest2, cop, p);

			glBindBuffer( GL_PIXEL_UNPACK_BUFFER_ARB, pbo_dest2);
			glBindTexture( GL_TEXTURE_2D, tex_screen);
			glTexSubImage2D( GL_TEXTURE_2D, 0, 0, 0, 
							WIDTH, HEIGHT, GL_RGB, GL_UNSIGNED_BYTE, NULL);

			displayImageFromTexture(2);

		#else
			// Left image
			glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
			glMatrixMode( GL_MODELVIEW);
			glLoadIdentity();
			glTranslated(cop.x-eyeDist, cop.y, cop.z);
			draw();

			pboUnregister(pbo_source);
			glBindBuffer( GL_PIXEL_PACK_BUFFER_ARB, pbo_source);
			glReadPixels( 0, 0, WIDTH, HEIGHT, GL_RGB, GL_UNSIGNED_BYTE, NULL);
			pboRegister(pbo_source);

			CUDATransformPixels(pbo_source, pbo_dest, cop, p);

			// Right image
			glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
			glMatrixMode( GL_MODELVIEW);
			glLoadIdentity();
			glTranslated(cop.x+eyeDist, cop.y, cop.z);
			draw();

			pboUnregister(pbo_source);
			glBindBuffer( GL_PIXEL_PACK_BUFFER_ARB, pbo_source);
			glReadPixels( 0, 0, WIDTH, HEIGHT, GL_RGB, GL_UNSIGNED_BYTE, NULL);
			pboRegister(pbo_source);

			CUDATransformPixels(pbo_source, pbo_dest2, cop, p);
		

			// Interlace the two images
			glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

			glBindBuffer( GL_PIXEL_UNPACK_BUFFER_ARB, pbo_dest);
			glBindTexture( GL_TEXTURE_2D, tex_screen);
			glTexSubImage2D( GL_TEXTURE_2D, 0, 0, 0, 
							WIDTH, HEIGHT, GL_RGB, GL_UNSIGNED_BYTE, NULL);

			displayImageFromTexture(1);

			glBindBuffer( GL_PIXEL_UNPACK_BUFFER_ARB, pbo_dest2);
			glBindTexture( GL_TEXTURE_2D, tex_screen);
			glTexSubImage2D( GL_TEXTURE_2D, 0, 0, 0, 
							WIDTH, HEIGHT, GL_RGB, GL_UNSIGNED_BYTE, NULL);

			displayImageFromTexture(0);
		#endif
	}
	else
	{
		glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
		glMatrixMode( GL_MODELVIEW);
		glLoadIdentity();
		glTranslated(cop.x, cop.y, cop.z);

		if(transform)
		{
			draw();
			
			pboUnregister(pbo_source);
			glBindBuffer( GL_PIXEL_PACK_BUFFER_ARB, pbo_source);
			glReadPixels( 0, 0, WIDTH, HEIGHT, GL_RGB, GL_UNSIGNED_BYTE, NULL);
			pboRegister(pbo_source);

			CUDATransformPixels(pbo_source, pbo_dest, cop, p);

			glBindBuffer( GL_PIXEL_UNPACK_BUFFER_ARB, pbo_dest);
			glBindTexture( GL_TEXTURE_2D, tex_screen);
			glTexSubImage2D( GL_TEXTURE_2D, 0, 0, 0, 
							WIDTH, HEIGHT, GL_RGB, GL_UNSIGNED_BYTE, NULL);

			displayImageFromTexture(2);
		}
		else
			draw();
	}

	//glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	glfwSwapBuffers();	
}



/****************************************************
Render scene using the parallel vertex transformation
****************************************************/
void transformationShaders(PF draw)
{
	float eyeDist = 0.05;

	if(stereo)
	{
		#if QUAD_STEREO
			glDrawBuffer(GL_BACK_LEFT);
            glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
			transformShader.bind();
			transformShader.uniform("P", p.x, p.y, p.z);
			transformShader.uniform("COP", cop.x-eyeDist, cop.y, cop.z);
			draw();
			transformShader.unbind();

			glDrawBuffer(GL_BACK_RIGHT);
            glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
			transformShader.bind();
			transformShader.uniform("P", p.x, p.y, p.z);
			transformShader.uniform("COP", cop.x+eyeDist, cop.y, cop.z);
			draw();
			transformShader.unbind();

		#else
			glEnable(GL_STENCIL_TEST);

			// Left image
			glStencilFunc(GL_NOTEQUAL, 1, 1);
			glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
			transformShader.bind();
			transformShader.uniform("P", p.x, p.y, p.z);
			transformShader.uniform("COP", cop.x-eyeDist, cop.y, cop.z);
			draw();
			transformShader.unbind();

			// Right image
			glStencilFunc(GL_EQUAL, 1, 1);
			transformShader.bind();
			transformShader.uniform("P", p.x, p.y, p.z);
			transformShader.uniform("COP", cop.x+eyeDist, cop.y, cop.z);
			draw();
			transformShader.unbind();
			
			glDisable(GL_STENCIL_TEST);
		#endif
	}
	else
	{
		glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
		if(transform)
		{
			transformShader.bind();
			transformShader.uniform("P", p.x, p.y, p.z);
			transformShader.uniform("COP", cop.x, cop.y, cop.z);
			draw();
			transformShader.unbind();
		}
		else
			draw();
	}

	//glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	glfwSwapBuffers();
}



/******************************************************
Render scene using the sequential pixel transformation
******************************************************/
void transformationCPU(PF draw)
{
	float eyeDist = 0.1;
	double tStart;
	if(stereo)
	{
		#if QUAD_STEREO
			glDrawBuffer(GL_BACK_LEFT);
            glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
			glLoadIdentity();
			glTranslated(cop.x-eyeDist, cop.y, cop.z);
			draw();
			glReadPixels(0,0,WIDTH,HEIGHT,GL_RGB, GL_UNSIGNED_BYTE, pixels_in);
			seqTransform(pixels_in, pixels_out, cop, p);
			glDrawPixels(WIDTH, HEIGHT, GL_RGB, GL_UNSIGNED_BYTE, pixels_out);

			glDrawBuffer(GL_BACK_RIGHT);
            glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
			glLoadIdentity();
			glTranslated(cop.x+eyeDist, cop.y, cop.z);
			draw();
			glReadPixels(0,0,WIDTH,HEIGHT,GL_RGB, GL_UNSIGNED_BYTE, pixels_in);
			seqTransform(pixels_in, pixels_out2, cop, p);
			glDrawPixels(WIDTH, HEIGHT, GL_RGB, GL_UNSIGNED_BYTE, pixels_out2);

		#else
			// Left image
			glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
			glLoadIdentity();
			glTranslated(cop.x-eyeDist, cop.y, cop.z);
			draw();
			glReadPixels(0,0,WIDTH,HEIGHT,GL_RGB, GL_UNSIGNED_BYTE, pixels_in);
			seqTransform(pixels_in, pixels_out, cop, p);

			// Right image
			glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
			glLoadIdentity();
			glTranslated(cop.x+eyeDist, cop.y, cop.z);
			draw();
			glReadPixels(0,0,WIDTH,HEIGHT,GL_RGB, GL_UNSIGNED_BYTE, pixels_in);
			seqTransform(pixels_in, pixels_out2, cop, p);
			
			// Interlace the two images
			glEnable(GL_STENCIL_TEST);
			glStencilFunc(GL_NOTEQUAL, 1, 1);
			glDrawPixels(WIDTH, HEIGHT, GL_RGB, GL_UNSIGNED_BYTE, pixels_out);
			
			glStencilFunc(GL_EQUAL, 1, 1);
			glDrawPixels(WIDTH, HEIGHT, GL_RGB, GL_UNSIGNED_BYTE, pixels_out2);
			glDisable(GL_STENCIL_TEST);
		#endif
	}
	else
	{
		glLoadIdentity();
		glTranslated(cop.x, cop.y, cop.z);

		glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
		draw();
		glReadPixels(0,0,WIDTH,HEIGHT,GL_RGB, GL_UNSIGNED_BYTE, pixels_in);
		tStart = glfwGetTime();
		seqTransform(pixels_in, pixels_out, cop, p);
		avgTransformTime += (glfwGetTime() - tStart);
		glDrawPixels(WIDTH, HEIGHT, GL_RGB, GL_UNSIGNED_BYTE, pixels_out);
	}

	//glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	glfwSwapBuffers();
}



/*******
MAIN
*******/
int main(int argc, char** argv) 
{
	cop.x = 0.0;
	cop.y = 0.0;
	cop.z = 1.0;
	p.x = 0.0;
	p.y = 0.0;
	p.z = 2.4142;

	init(0,1);
	mainLoop();
	shutDown(0);

	return 0;
}