#ifndef MAIN_H
#define MAIN_H

#include "models.h"
#include "cudaDevice.h"
#include "cudaTransform.h"
#include "sequentialTransform.h"
#include "shader.h"

// Pointer to function definition
typedef void (*PF)();

// Main functions
void init(int stereoMode, int transformation);
void shutDown(int returnCode);
void mainLoop();

// Transformation functions
void transformationCUDA(PF draw);
void transformationShaders(PF draw);
void transformationCPU(PF draw);

// StencilMask related functions
void glEnable2D();
void glDisable2D();
void initStencilMask();

// PBO related functions
void createPBO( GLuint* pbo);
void deletePBO( GLuint* pbo);
void createTexture(GLuint* tex_name, unsigned int size_x, unsigned int size_y);
void deleteTexture( GLuint* tex);
void displayImageFromTexture(int stencilState);


static Shader transformShader;
bool running;
bool stereo;
bool transform;
double avgTransformTime;


// PBO variables
GLuint pbo_source;
GLuint pbo_dest;
GLuint pbo_dest2;
unsigned int size_tex_data;
unsigned int num_texels;
unsigned int num_values;

// render target
GLuint tex_screen;


// Eye position, COP, and Projector position, P
struct point3 cop, p;


unsigned char *pixels_in,	// Original image
			  *pixels_out,	// Transformed image
			  *pixels_out2;	// for stereo vision


#endif // MAIN_H