#include "structs.h"
#include "sequentialTransform.h"


void seqTransform(unsigned char* in_pixels, unsigned char* out_pixels, struct point3 COP, struct point3 P)
{
	seqCylinderTransform(in_pixels, out_pixels, COP, P);
	seqSphereTransform(in_pixels, out_pixels, COP, P);
}

void seqCylinderTransform(unsigned char* in_pixels, unsigned char* out_pixels, struct point3 COP, struct point3 P)
{
	//input:
	// COP
	float xo = COP.x;
	float yo = COP.y;
	float zo = COP.z;

	// P
	float xj = P.x;
	float yj = P.y;
	float zj = P.z;

	float R2 = WIDTH_SCALE_FACTOR * WIDTH_SCALE_FACTOR;
	float halfHeight = HEIGHT/2.0;
	float halfWidth = WIDTH/2.0;

	// For stepping in the x and y directions
	float deltaXP = 2.0 * WIDTH_SCALE_FACTOR / WIDTH;
	float deltaYP = 2.0 * HEIGHT_SCALE_FACTOR /HEIGHT;

	// Where projection intersects the planar surface
	float xp = (-halfWidth  * 2.0 * WIDTH_SCALE_FACTOR  /WIDTH );
	float yp = (-halfHeight * 2.0 * HEIGHT_SCALE_FACTOR /HEIGHT);
	float zp = 0;

	int u,v;
	float xj_xp, zj_zp, xj_xp2, zj_zp2, xpzj,
		  xjzp, a, b, c, z, x, y, xc, yc;
	
	for(int i=0; i<WIDTH; i++)
	{
		// Simplifying common expressions
		xj_xp = xj - xp;
		zj_zp = zj - zp;
		  
		xj_xp2 = xj_xp * xj_xp;
		zj_zp2 = zj_zp * zj_zp;

		xpzj = xp * zj;
		xjzp = xj * zp;

		// Solve z-equation
		a = xj_xp2 + zj_zp2;
		b = 2.0 * xj_xp * (xpzj - xjzp);
		c = (xpzj - xjzp) * (xpzj - xjzp) - R2 * zj_zp2;
		
		z = (-b - sqrt(b * b - 4 * a * c)) / (2 * a);

		// Solve for x and xc, and decide u
		x = (xpzj - xjzp + z * xj_xp) / zj_zp;
		xc = (x *(zo - zp) + xo * (zp - z)) / (zo - z);

		u = halfWidth * (xc / WIDTH_SCALE_FACTOR + 1);

		for( int j =0; j<halfHeight; j++)
		{	
			// Solve for y and yc, and decide v
			y = (yp - yj) * sqrt(((zj - z) * (zj - z) + (x - xj) * (x - xj)) / (zj_zp2 + xj_xp2)) + yj;
			yc = (y - yo) * sqrt(((zo - zp) * (zo - zp) + (xo - xc) * (xo - xc)) / ((zo - z) * (zo - z) + (xo - x) * (xo - x))) + yo;

			v = halfHeight * (yc / HEIGHT_SCALE_FACTOR + 1);

			// Put pixel value in out_pixels
			if(u < 0 || u >= WIDTH || v < 0 || v >= HEIGHT)
			{
				out_pixels[(i + j * WIDTH) * NUM_COLOR_FACTOR    ] = 0x55;
				out_pixels[(i + j * WIDTH) * NUM_COLOR_FACTOR + 1] = 0x55;
				out_pixels[(i + j * WIDTH) * NUM_COLOR_FACTOR + 2] = 0x55;
			}
			else
			{
				out_pixels[(i + j * WIDTH) * NUM_COLOR_FACTOR    ] = in_pixels[(u + v * WIDTH) * NUM_COLOR_FACTOR    ];
				out_pixels[(i + j * WIDTH) * NUM_COLOR_FACTOR + 1] = in_pixels[(u + v * WIDTH) * NUM_COLOR_FACTOR + 1];
				out_pixels[(i + j * WIDTH) * NUM_COLOR_FACTOR + 2] = in_pixels[(u + v * WIDTH) * NUM_COLOR_FACTOR + 2];
			}
			yp += deltaYP;
		}
		xp += deltaXP;
		yp = (-halfHeight * 2.0 * HEIGHT_SCALE_FACTOR /HEIGHT);
	}
}

void seqSphereTransform(unsigned char* in_pixels, unsigned char* out_pixels, struct point3 COP, struct point3 P)
{
	//input:
	// COP
	float xo = COP.x;
	float yo = COP.y;
	float zo = COP.z;

	// P
	float xj = P.x;
	float yj = P.y;
	float zj = P.z;

	float R2 = WIDTH_SCALE_FACTOR * WIDTH_SCALE_FACTOR;
	float halfHeight = HEIGHT/2.0;
	float halfWidth = WIDTH/2.0;

	// For stepping in the x and y directions
	float deltaXP = 2.0 * WIDTH_SCALE_FACTOR / WIDTH;
	float deltaYP = 2.0 * HEIGHT_SCALE_FACTOR /HEIGHT;

	// Where projection intersects the planar surface
	float xp = (-halfWidth  * 2.0 * WIDTH_SCALE_FACTOR  /WIDTH );
	float yp = 0;
	float zp = 0;

	int u,v;
	float xj_xp, yj_yp, zj_zp, xj_xp2, yj_yp2, zj_zp2, xj2, xp2, yj2, yp2, zj2, zp2,
		  xpxj, xpzj, xjzp, xpyj, xjyp, ypyj, zpzj, a, b, d, z, x, y, xc, yc;
	
	for(int i=0; i<WIDTH; i++)
	{
		// Simplifying common expressions
		xj_xp = xj - xp;
		zj_zp = zj - zp;
		  
		xj_xp2 = xj_xp * xj_xp;
		zj_zp2 = zj_zp * zj_zp;
		  
		xj2 = xj * xj;
		xp2 = xp * xp;
		zj2 = zj * zj;
		zp2 = zp * zp;

		xpxj = xp * xj;
		xpzj = xp * zj;
		xjzp = xj * zp;
		zpzj = zp * zj;

		for( int j =halfHeight; j<HEIGHT; j++)
		{
			// Simplifying more common expressions
			yj_yp = yj - yp;
			yj_yp2 = yj_yp * yj_yp;

			yj2 = yj * yj;
			yp2 = yp * yp;

			xjyp = xj * yp;
			xpyj = xp * yj;
			ypyj = yp * yj;

			// Solve z-equation
			a = xj_xp2 + yj_yp2 + zj_zp2;
			b = zj * (xp2 - xpxj - yp * yj_yp)
			    + zp * (xj2 - xpxj + yj * yj_yp);
			d = zj_zp2 * (zj2 * (R2 - xp2 - yp2) - 2 * zpzj * (R2 - xpxj - ypyj)
				- (xjyp - xpyj) * (xjyp - xpyj) + zp2 * (R2 - xj2 - yj2)
				+ R2 * (xj_xp2 + yj_yp2));
			
			z = (b - sqrt(d)) / a;

			// Solve for x and xc, and decide u
			x = (xpzj - xjzp + z * xj_xp) / zj_zp;
			xc = (x *(zo - zp) + xo * (zp - z)) / (zo - z);

			u = halfWidth * (xc / WIDTH_SCALE_FACTOR + 1);
	
			// Solve for y and yc, and decide v
			y = (yp - yj) * sqrt(((zj - z) * (zj - z) + (x - xj) * (x - xj)) / (zj_zp2 + xj_xp2)) + yj;
			yc = (y - yo) * sqrt(((zo - zp) * (zo - zp) + (xo - xc) * (xo - xc)) / ((zo - z) * (zo - z) + (xo - x) * (xo - x))) + yo;

			v = halfHeight * (yc / HEIGHT_SCALE_FACTOR + 1);

			// Put pixel value in out_pixels
			if(u < 0 || u >= WIDTH || v < 0 || v >= HEIGHT
			   || (j > halfHeight && sqrt((i - halfWidth) * (i - halfWidth) + (j - halfHeight) * (j - halfHeight)) > halfHeight))
			{
				out_pixels[(i + j * WIDTH) * NUM_COLOR_FACTOR    ] = 0x55;
				out_pixels[(i + j * WIDTH) * NUM_COLOR_FACTOR + 1] = 0x55;
				out_pixels[(i + j * WIDTH) * NUM_COLOR_FACTOR + 2] = 0x55;
			}
			else
			{
				out_pixels[(i + j * WIDTH) * NUM_COLOR_FACTOR    ] = in_pixels[(u + v * WIDTH) * NUM_COLOR_FACTOR    ];
				out_pixels[(i + j * WIDTH) * NUM_COLOR_FACTOR + 1] = in_pixels[(u + v * WIDTH) * NUM_COLOR_FACTOR + 1];
				out_pixels[(i + j * WIDTH) * NUM_COLOR_FACTOR + 2] = in_pixels[(u + v * WIDTH) * NUM_COLOR_FACTOR + 2];
			}
			yp += deltaYP;
		}
		xp += deltaXP;
		yp = 0;
	}
}