#include "shader.h"
#include <cstdio>
#include <cstring>
#include <cstdlib>

static void infoLog(GLhandleARB obj, const char *name) {
	int length = 0;
	int written  = 0;
	char *text;
	
	glGetObjectParameterivARB(obj, GL_OBJECT_INFO_LOG_LENGTH_ARB, &length);
	
	if(length > 0) {
		text = new char[length];
		glGetInfoLogARB(obj, length, &written, text);
		//FILE *file = fopen("logfile.txt", "w");
		FILE *file = stdout;
		fprintf(file, "Shader '%s': %s\n\n", name, text);
		//fclose(file);
		delete [] text;
	}
}

bool Shader::load(const char *vertname, const char *fragname) {
	char *vertsrc = 0, *fragsrc = 0;
	int size;
	FILE *file;
	
	file = fopen(vertname, "rb");
	if(file) {
		fseek(file, 0, SEEK_END);
		size = ftell(file);
		rewind(file);
		vertsrc = new char[size + 1];
		fread(vertsrc, size, 1, file);
		vertsrc[size] = 0;
		fclose(file);
	}
	
	file = fopen(fragname, "rb");
	if(file) {
		fseek(file, 0, SEEK_END);
		size = ftell(file);
		rewind(file);
		fragsrc = new char[size + 1];
		fread(fragsrc, size, 1, file);
		fragsrc[size] = 0;
		fclose(file);
	}
	
	if(!vertsrc && !fragsrc) {
		return false;
	}
	
	program = glCreateProgramObjectARB();
	
	if(vertsrc) {
		vertex = glCreateShaderObjectARB(GL_VERTEX_SHADER_ARB);
		glShaderSourceARB(vertex, 1, (const char **)&vertsrc, 0);
		glCompileShaderARB(vertex);
		glAttachObjectARB(program, vertex);
	}
	if(fragsrc) {
		fragment = glCreateShaderObjectARB(GL_FRAGMENT_SHADER_ARB);
		glShaderSourceARB(fragment, 1, (const char **)&fragsrc, 0);
		glCompileShaderARB(fragment);
		glAttachObjectARB(program, fragment);
	}
	
	glLinkProgramARB(program);
	
	delete [] vertsrc;
	delete [] fragsrc;
	
	infoLog(program, vertname);

	return true;
}

void Shader::bind() {
	glUseProgramObjectARB(program);
}

void Shader::unbind() {
	glUseProgramObjectARB(0);
}

void Shader::uniform(const char *name, int a) {
	glUniform1iARB(glGetUniformLocationARB(program, name), a);
}

void Shader::uniform(const char *name, int a, int b) {
	glUniform2iARB(glGetUniformLocationARB(program, name), a, b);
}

void Shader::uniform(const char *name, int a, int b, int c) {
	glUniform3iARB(glGetUniformLocationARB(program, name), a, b, c);
}

void Shader::uniform(const char *name, int a, int b, int c, int d) {
	glUniform4iARB(glGetUniformLocationARB(program, name), a, b, c, d);
}

void Shader::uniform(const char *name, float a) {
	glUniform1fARB(glGetUniformLocationARB(program, name), a);
}

void Shader::uniform(const char *name, float a, float b) {
	glUniform2fARB(glGetUniformLocationARB(program, name), a, b);
}

void Shader::uniform(const char *name, float a, float b, float c) {
	glUniform3fARB(glGetUniformLocationARB(program, name), a, b, c);
}

void Shader::uniform(const char *name, float a, float b, float c, float d) {
	glUniform4fARB(glGetUniformLocationARB(program, name), a, b, c, d);
}

void Shader::uniform(const char *name, const GLfloat *v) {
	glUniformMatrix4fv(glGetUniformLocationARB(program, name), 1, GL_FALSE, v);
}

void Shader::attribute(const char *name, float *v) {
	GLint loc = glGetAttribLocationARB(program, name);
	glEnableVertexAttribArrayARB(loc);
	glVertexAttribPointerARB(loc, 1, GL_FLOAT, 0, 0, v);
}

void Shader::attribute(const char *name, GLuint comps, GLenum type, void *v) {
	GLint loc = glGetAttribLocationARB(program, name);
	glEnableVertexAttribArrayARB(loc);
	glVertexAttribPointerARB(loc, comps, type, 0, 0, v);
}